@@ -1,19 +1,22 @@
 # mypy: disable-error-code="attr-defined"
 import pytest
 import numpy as np
-from numpy import cos, sin, pi
-from numpy.testing import (assert_equal, assert_almost_equal, assert_allclose,
-                           assert_, suppress_warnings)
+from numpy.testing import assert_equal, assert_almost_equal, assert_allclose
 from hypothesis import given
 import hypothesis.strategies as st
 import hypothesis.extra.numpy as hyp_num
 
-from scipy.integrate import (quadrature, romberg, romb, newton_cotes,
+from scipy.integrate import (romb, newton_cotes,
                              cumulative_trapezoid, trapezoid,
-                             quad, simpson, fixed_quad, AccuracyWarning,
+                             quad, simpson, fixed_quad,
                              qmc_quad, cumulative_simpson)
 from scipy.integrate._quadrature import _cumulative_simpson_unequal_intervals
-from scipy import stats, special
+
+from scipy import stats, special, integrate
+from scipy.conftest import array_api_compatible, skip_xp_invalid_arg
+from scipy._lib._array_api_no_0d import xp_assert_close
+
+skip_xp_backends = pytest.mark.skip_xp_backends
 
 
 class TestFixedQuad:
@@ -32,59 +35,10 @@ class TestFixedQuad:
         assert_allclose(got, expected, rtol=1e-12)
 
 
-@pytest.mark.filterwarnings('ignore::DeprecationWarning')
 class TestQuadrature:
     def quad(self, x, a, b, args):
         raise NotImplementedError
 
-    def test_quadrature(self):
-        # Typical function with two extra arguments:
-        def myfunc(x, n, z):       # Bessel function integrand
-            return cos(n*x-z*sin(x))/pi
-        val, err = quadrature(myfunc, 0, pi, (2, 1.8))
-        table_val = 0.30614353532540296487
-        assert_almost_equal(val, table_val, decimal=7)
-
-    def test_quadrature_rtol(self):
-        def myfunc(x, n, z):       # Bessel function integrand
-            return 1e90 * cos(n*x-z*sin(x))/pi
-        val, err = quadrature(myfunc, 0, pi, (2, 1.8), rtol=1e-10)
-        table_val = 1e90 * 0.30614353532540296487
-        assert_allclose(val, table_val, rtol=1e-10)
-
-    def test_quadrature_miniter(self):
-        # Typical function with two extra arguments:
-        def myfunc(x, n, z):       # Bessel function integrand
-            return cos(n*x-z*sin(x))/pi
-        table_val = 0.30614353532540296487
-        for miniter in [5, 52]:
-            val, err = quadrature(myfunc, 0, pi, (2, 1.8), miniter=miniter)
-            assert_almost_equal(val, table_val, decimal=7)
-            assert_(err < 1.0)
-
-    def test_quadrature_single_args(self):
-        def myfunc(x, n):
-            return 1e90 * cos(n*x-1.8*sin(x))/pi
-        val, err = quadrature(myfunc, 0, pi, args=2, rtol=1e-10)
-        table_val = 1e90 * 0.30614353532540296487
-        assert_allclose(val, table_val, rtol=1e-10)
-
-    def test_romberg(self):
-        # Typical function with two extra arguments:
-        def myfunc(x, n, z):       # Bessel function integrand
-            return cos(n*x-z*sin(x))/pi
-        val = romberg(myfunc, 0, pi, args=(2, 1.8))
-        table_val = 0.30614353532540296487
-        assert_almost_equal(val, table_val, decimal=7)
-
-    def test_romberg_rtol(self):
-        # Typical function with two extra arguments:
-        def myfunc(x, n, z):       # Bessel function integrand
-            return 1e19*cos(n*x-z*sin(x))/pi
-        val = romberg(myfunc, 0, pi, args=(2, 1.8), rtol=1e-10)
-        table_val = 1e19*0.30614353532540296487
-        assert_allclose(val, table_val, rtol=1e-10)
-
     def test_romb(self):
         assert_equal(romb(np.arange(17)), 128)
 
@@ -96,19 +50,6 @@ class TestQuadrature:
         val2, err = quad(lambda x: np.cos(0.2*x), x.min(), x.max())
         assert_allclose(val, val2, rtol=1e-8, atol=0)
 
-        # should be equal to romb with 2**k+1 samples
-        with suppress_warnings() as sup:
-            sup.filter(AccuracyWarning, "divmax .4. exceeded")
-            val3 = romberg(lambda x: np.cos(0.2*x), x.min(), x.max(), divmax=4)
-        assert_allclose(val, val3, rtol=1e-12, atol=0)
-
-    def test_non_dtype(self):
-        # Check that we work fine with functions returning float
-        import math
-        valmath = romberg(math.sin, 0, 1)
-        expected_val = 0.45969769413185085
-        assert_almost_equal(valmath, expected_val, decimal=7)
-
     def test_newton_cotes(self):
         """Test the first few degrees, for evenly spaced points."""
         n = 1
@@ -239,13 +180,6 @@ class TestQuadrature:
         assert_equal(result, expected)
 
 
-@pytest.mark.parametrize('func', [romberg, quadrature])
-def test_deprecate_integrator(func):
-    message = f"`scipy.integrate.{func.__name__}` is deprecated..."
-    with pytest.deprecated_call(match=message):
-        func(np.exp, 0, 1)
-
-
 class TestCumulative_trapezoid:
     def test_1d(self):
         x = np.linspace(-2, 2, num=5)
@@ -324,63 +258,94 @@ class TestCumulative_trapezoid:
     @pytest.mark.parametrize(
         "initial", [1, 0.5]
     )
-    def test_initial_warning(self, initial):
+    def test_initial_error(self, initial):
         """If initial is not None or 0, a ValueError is raised."""
         y = np.linspace(0, 10, num=10)
-        with pytest.deprecated_call(match="`initial`"):
-            res = cumulative_trapezoid(y, initial=initial)
-        assert_allclose(res, [initial, *np.cumsum(y[1:] + y[:-1])/2])
+        with pytest.raises(ValueError, match="`initial`"):
+            cumulative_trapezoid(y, initial=initial)
 
     def test_zero_len_y(self):
         with pytest.raises(ValueError, match="At least one point is required"):
             cumulative_trapezoid(y=[])
 
 
+@array_api_compatible
 class TestTrapezoid:
-    def test_simple(self):
-        x = np.arange(-10, 10, .1)
-        r = trapezoid(np.exp(-.5 * x ** 2) / np.sqrt(2 * np.pi), dx=0.1)
+    def test_simple(self, xp):
+        x = xp.arange(-10, 10, .1)
+        r = trapezoid(xp.exp(-.5 * x ** 2) / xp.sqrt(2 * xp.asarray(xp.pi)), dx=0.1)
         # check integral of normal equals 1
-        assert_allclose(r, 1)
+        xp_assert_close(r, xp.asarray(1.0))
 
-    def test_ndim(self):
-        x = np.linspace(0, 1, 3)
-        y = np.linspace(0, 2, 8)
-        z = np.linspace(0, 3, 13)
+    @skip_xp_backends('jax.numpy',
+                      reasons=["JAX arrays do not support item assignment"])
+    @pytest.mark.usefixtures("skip_xp_backends")
+    def test_ndim(self, xp):
+        x = xp.linspace(0, 1, 3)
+        y = xp.linspace(0, 2, 8)
+        z = xp.linspace(0, 3, 13)
 
-        wx = np.ones_like(x) * (x[1] - x[0])
+        wx = xp.ones_like(x) * (x[1] - x[0])
         wx[0] /= 2
         wx[-1] /= 2
-        wy = np.ones_like(y) * (y[1] - y[0])
+        wy = xp.ones_like(y) * (y[1] - y[0])
         wy[0] /= 2
         wy[-1] /= 2
-        wz = np.ones_like(z) * (z[1] - z[0])
+        wz = xp.ones_like(z) * (z[1] - z[0])
         wz[0] /= 2
         wz[-1] /= 2
 
         q = x[:, None, None] + y[None,:, None] + z[None, None,:]
 
-        qx = (q * wx[:, None, None]).sum(axis=0)
-        qy = (q * wy[None, :, None]).sum(axis=1)
-        qz = (q * wz[None, None, :]).sum(axis=2)
+        qx = xp.sum(q * wx[:, None, None], axis=0)
+        qy = xp.sum(q * wy[None, :, None], axis=1)
+        qz = xp.sum(q * wz[None, None, :], axis=2)
 
         # n-d `x`
         r = trapezoid(q, x=x[:, None, None], axis=0)
-        assert_allclose(r, qx)
+        xp_assert_close(r, qx)
         r = trapezoid(q, x=y[None,:, None], axis=1)
-        assert_allclose(r, qy)
+        xp_assert_close(r, qy)
         r = trapezoid(q, x=z[None, None,:], axis=2)
-        assert_allclose(r, qz)
+        xp_assert_close(r, qz)
 
         # 1-d `x`
         r = trapezoid(q, x=x, axis=0)
-        assert_allclose(r, qx)
+        xp_assert_close(r, qx)
         r = trapezoid(q, x=y, axis=1)
-        assert_allclose(r, qy)
+        xp_assert_close(r, qy)
         r = trapezoid(q, x=z, axis=2)
-        assert_allclose(r, qz)
+        xp_assert_close(r, qz)
 
-    def test_masked(self):
+    @skip_xp_backends('jax.numpy',
+                      reasons=["JAX arrays do not support item assignment"])
+    @pytest.mark.usefixtures("skip_xp_backends")
+    def test_gh21908(self, xp):
+        # extended testing for n-dim arrays
+        x = xp.reshape(xp.linspace(0, 29, 30), (3, 10))
+        y = xp.reshape(xp.linspace(0, 29, 30), (3, 10))
+
+        out0 = xp.linspace(200, 380, 10)
+        xp_assert_close(trapezoid(y, x=x, axis=0), out0)
+        xp_assert_close(trapezoid(y, x=xp.asarray([0, 10., 20.]), axis=0), out0)
+        # x needs to be broadcastable against y
+        xp_assert_close(
+            trapezoid(y, x=xp.asarray([0, 10., 20.])[:, None], axis=0),
+            out0
+        )
+        with pytest.raises(Exception):
+            # x is not broadcastable against y
+            trapezoid(y, x=xp.asarray([0, 10., 20.])[None, :], axis=0)
+
+        out1 = xp.asarray([ 40.5, 130.5, 220.5])
+        xp_assert_close(trapezoid(y, x=x, axis=1), out1)
+        xp_assert_close(
+            trapezoid(y, x=xp.linspace(0, 9, 10), axis=1),
+            out1
+        )
+
+    @skip_xp_invalid_arg
+    def test_masked(self, xp):
         # Testing that masked arrays behave as if the function is 0 where
         # masked
         x = np.arange(5)
@@ -396,8 +361,21 @@ class TestTrapezoid:
         xm = np.ma.array(x, mask=mask)
         assert_allclose(trapezoid(y, xm), r)
 
+    @skip_xp_backends(np_only=True,
+                      reasons=['array-likes only supported for NumPy backend'])
+    @pytest.mark.usefixtures("skip_xp_backends")
+    def test_array_like(self, xp):
+        x = list(range(5))
+        y = [t * t for t in x]
+        xarr = xp.asarray(x, dtype=xp.float64)
+        yarr = xp.asarray(y, dtype=xp.float64)
+        res = trapezoid(y, x)
+        resarr = trapezoid(yarr, xarr)
+        xp_assert_close(res, resarr)
+
 
 class TestQMCQuad:
+    @pytest.mark.thread_unsafe
     def test_input_validation(self):
         message = "`func` must be callable."
         with pytest.raises(TypeError, match=message):
@@ -434,8 +412,9 @@ class TestQMCQuad:
         with pytest.raises(TypeError, match=message):
             qmc_quad(lambda x: 1, [0, 0], [1, 1], log=10)
 
-    def basic_test(self, n_points=2**8, n_estimates=8, signs=np.ones(2)):
-
+    def basic_test(self, n_points=2**8, n_estimates=8, signs=None):
+        if signs is None:
+            signs = np.ones(2)
         ndim = 2
         mean = np.zeros(ndim)
         cov = np.eye(ndim)
@@ -473,6 +452,7 @@ class TestQMCQuad:
     def test_sign(self, signs):
         self.basic_test(signs=signs)
 
+    @pytest.mark.thread_unsafe
     @pytest.mark.parametrize("log", [False, True])
     def test_zero(self, log):
         message = "A lower limit was equal to an upper limit, so"
@@ -655,6 +635,7 @@ class TestCumulativeSimpson:
         # `simpson` uses the trapezoidal rule
         return theoretical_difference
 
+    @pytest.mark.thread_unsafe
     @pytest.mark.slow
     @given(
         y=hyp_num.arrays(
@@ -682,9 +663,10 @@ class TestCumulativeSimpson:
             y, x=np.arange(y.shape[-1])
         )
         np.testing.assert_allclose(
-            res[..., 1:], ref[..., 1:] + theoretical_difference[..., 1:]
+            res[..., 1:], ref[..., 1:] + theoretical_difference[..., 1:], atol=1e-16
         )
 
+    @pytest.mark.thread_unsafe
     @pytest.mark.slow
     @given(
         y=hyp_num.arrays(
@@ -719,3 +701,32 @@ class TestCumulativeSimpson:
         np.testing.assert_allclose(
             res[..., 1:], ref[..., 1:] + theoretical_difference[..., 1:]
         )
+
+class TestLebedev:
+    def test_input_validation(self):
+        # only certain rules are available
+        message = "Order n=-1 not available..."
+        with pytest.raises(NotImplementedError, match=message):
+            integrate.lebedev_rule(-1)
+
+    def test_quadrature(self):
+        # Test points/weights to integrate an example function
+
+        def f(x):
+            return np.exp(x[0])
+
+        x, w = integrate.lebedev_rule(15)
+        res = w @ f(x)
+        ref = 14.7680137457653  # lebedev_rule reference [3]
+        assert_allclose(res, ref, rtol=1e-14)
+        assert_allclose(np.sum(w), 4 * np.pi)
+
+    @pytest.mark.parametrize('order', list(range(3, 32, 2)) + list(range(35, 132, 6)))
+    def test_properties(self, order):
+        x, w = integrate.lebedev_rule(order)
+        # dispersion should be maximal; no clear spherical mean
+        with np.errstate(divide='ignore', invalid='ignore'):
+            res = stats.directional_stats(x.T, axis=0)
+            assert_allclose(res.mean_resultant_length, 0, atol=1e-15)
+        # weights should sum to 4*pi (surface area of unit sphere)
+        assert_allclose(np.sum(w), 4*np.pi)
