@@ -59,7 +59,8 @@ map_coordinate(double in, npy_intp len, int mode)
                 npy_intp sz2 = 2 * len;
                 if (in < -sz2)
                     in = sz2 * (npy_intp)(-in / sz2) + in;
-                in = in < -len ? in + sz2 : -in - 1;
+                // -1e-15 check to avoid possibility that: (-in - 1) == -1
+                in = in < -len ? in + sz2 : (in > -1e-15 ? 1e-15 : -in) - 1;
             }
             break;
         case NI_EXTEND_WRAP:
@@ -72,6 +73,14 @@ map_coordinate(double in, npy_intp len, int mode)
                 in += sz * ((npy_intp)(-in / sz) + 1);
             }
             break;
+        case NI_EXTEND_GRID_WRAP:
+            if (len <= 1) {
+                in = 0;
+            } else {
+                // in = len - 1 + fmod(in + 1, len);
+                in += len * ((npy_intp)((-1 - in) / len) + 1);
+            }
+            break;
         case NI_EXTEND_NEAREST:
             in = 0;
             break;
@@ -109,6 +118,13 @@ map_coordinate(double in, npy_intp len, int mode)
                 in -= sz * (npy_intp)(in / sz);
             }
             break;
+        case NI_EXTEND_GRID_WRAP:
+            if (len <= 1) {
+                in = 0;
+            } else {
+                in -= len * (npy_intp)(in / len);
+            }
+            break;
         case NI_EXTEND_NEAREST:
             in = len - 1;
             break;
@@ -227,14 +243,25 @@ case NPY_##_TYPE:                                    \
     *(_type *)_po = (_type)_t;                       \
     break
 
+int _get_spline_boundary_mode(int mode)
+{
+    if ((mode == NI_EXTEND_CONSTANT) || (mode == NI_EXTEND_WRAP))
+        // Modes without an analytic prefilter or explicit prepadding use
+        // mirror extension.
+        return NI_EXTEND_MIRROR;
+    return mode;
+}
+
 int
 NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
                 int, int, void*), void* map_data, PyArrayObject* matrix_ar,
                 PyArrayObject* shift_ar, PyArrayObject *coordinates,
-                PyArrayObject *output, int order, int mode, double cval)
+                PyArrayObject *output, int order, int mode, double cval,
+                int nprepad)
 {
     char *po, *pi, *pc = NULL;
     npy_intp **edge_offsets = NULL, **data_offsets = NULL, filter_size;
+    char **edge_grid_const = NULL;
     npy_intp ftmp[NPY_MAXDIMS], *fcoordinates = NULL, *foffsets = NULL;
     npy_intp cstride = 0, kk, hh, ll, jj;
     npy_intp size;
@@ -243,7 +270,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
     NI_Iterator io, ic;
     npy_double *matrix = matrix_ar ? (npy_double*)PyArray_DATA(matrix_ar) : NULL;
     npy_double *shift = shift_ar ? (npy_double*)PyArray_DATA(shift_ar) : NULL;
-    int irank = 0, orank;
+    int irank = 0, orank, spline_mode;
     NPY_BEGIN_THREADS_DEF;
 
     NPY_BEGIN_THREADS;
@@ -274,6 +301,28 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
         PyErr_NoMemory();
         goto exit;
     }
+
+    if (mode == NI_EXTEND_GRID_CONSTANT) {
+        // boolean indicating if the current point in the filter footprint is
+        // outside the bounds
+        edge_grid_const = malloc(irank * sizeof(char*));
+        if (NPY_UNLIKELY(!edge_grid_const)) {
+            NPY_END_THREADS;
+            PyErr_NoMemory();
+            goto exit;
+        }
+        for(jj = 0; jj < irank; jj++)
+            edge_grid_const[jj] = NULL;
+        for(jj = 0; jj < irank; jj++) {
+            edge_grid_const[jj] = malloc((order + 1) * sizeof(char));
+            if (NPY_UNLIKELY(!edge_grid_const[jj])) {
+                NPY_END_THREADS;
+                PyErr_NoMemory();
+                goto exit;
+            }
+        }
+    }
+
     for(jj = 0; jj < irank; jj++)
         data_offsets[jj] = NULL;
     for(jj = 0; jj < irank; jj++) {
@@ -342,11 +391,21 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
         }
     }
 
+    spline_mode = _get_spline_boundary_mode(mode);
+
     size = PyArray_SIZE(output);
     for(kk = 0; kk < size; kk++) {
         double t = 0.0;
         int constant = 0, edge = 0;
         npy_intp offset = 0;
+        if (mode == NI_EXTEND_GRID_CONSTANT) {
+            // reset edge flags for each location in the filter footprint
+            for (hh = 0; hh < irank; hh++) {
+                for(ll = 0; ll <= order; ll++) {
+                    edge_grid_const[hh][ll] = 0;
+                }
+            }
+        }
         if (map) {
             NPY_END_THREADS;
             /* call mappint functions: */
@@ -403,11 +462,15 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
                 goto exit;
             }
         }
+
         /* iterate over axes: */
         for(hh = 0; hh < irank; hh++) {
-            /* if the input coordinate is outside the borders, map it: */
-            double cc = map_coordinate(icoor[hh], idimensions[hh], mode);
-            if (cc > -1.0) {
+            double cc = icoor[hh] + nprepad;
+            if ((mode != NI_EXTEND_GRID_CONSTANT) && (mode != NI_EXTEND_NEAREST)) {
+                /* if the input coordinate is outside the borders, map it: */
+                cc = map_coordinate(cc, idimensions[hh], mode);
+            }
+            if (cc > -1.0 || mode == NI_EXTEND_GRID_CONSTANT || mode == NI_EXTEND_NEAREST) {
                 /* find the filter location along this axis: */
                 npy_intp start;
                 if (order & 1) {
@@ -417,32 +480,33 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
                 }
                 /* get the offset to the start of the filter: */
                 offset += istrides[hh] * start;
-                if (start < 0 || start + order >= idimensions[hh]) {
-                    /* implement border mapping, if outside border: */
-                    edge = 1;
-                    edge_offsets[hh] = data_offsets[hh];
+                npy_intp idx = 0;
+
+                if (mode == NI_EXTEND_GRID_CONSTANT) {
+                    // Determine locations in the filter footprint that are
+                    // outside the range.
                     for(ll = 0; ll <= order; ll++) {
-                        npy_intp idx = start + ll;
-                        npy_intp len = idimensions[hh];
-                        if (len <= 1) {
-                            idx = 0;
-                        } else {
-                            npy_intp s2 = 2 * len - 2;
-                            if (idx < 0) {
-                                idx = s2 * (-idx / s2) + idx;
-                                idx = idx <= 1 - len ? idx + s2 : -idx;
-                            } else if (idx >= len) {
-                                idx -= s2 * (idx / s2);
-                                if (idx >= len)
-                                    idx = s2 - idx;
-                            }
-                        }
-                        /* calculate and store the offests at this edge: */
-                        edge_offsets[hh][ll] = istrides[hh] * (idx - start);
+                        idx = start + ll;
+                        edge_grid_const[hh][ll] = (idx < 0 || idx >= idimensions[hh]);
                     }
                 } else {
-                    /* we are not at the border, use precalculated offsets: */
-                    edge_offsets[hh] = NULL;
+
+                    if (start < 0 || start + order >= idimensions[hh]) {
+                        /* implement border mapping, if outside border: */
+                        edge = 1;
+                        edge_offsets[hh] = data_offsets[hh];
+
+                        for(ll = 0; ll <= order; ll++) {
+                            idx = start + ll;
+                            idx = (npy_intp)map_coordinate(idx, idimensions[hh], spline_mode);
+
+                            /* calculate and store the offsets at this edge: */
+                            edge_offsets[hh][ll] = istrides[hh] * (idx - start);
+                        }
+                    } else {
+                        /* we are not at the border, use precalculated offsets: */
+                        edge_offsets[hh] = NULL;
+                    }
                 }
                 get_spline_interpolation_weights(cc, order, splvals[hh]);
             } else {
@@ -459,50 +523,61 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
             for(hh = 0; hh < filter_size; hh++) {
                 double coeff = 0.0;
                 npy_intp idx = 0;
-
-                if (NPY_UNLIKELY(edge)) {
+                char is_cval = 0;
+                if (mode == NI_EXTEND_GRID_CONSTANT) {
                     for(ll = 0; ll < irank; ll++) {
-                        if (edge_offsets[ll])
-                            idx += edge_offsets[ll][ff[ll]];
-                        else
-                            idx += ff[ll] * istrides[ll];
+                        if (edge_grid_const[ll][ff[ll]]) {
+                            is_cval = 1;
+                        }
                     }
-                } else {
-                    idx = foffsets[hh];
                 }
-                idx += offset;
-                switch (type_num) {
-                    CASE_INTERP_COEFF(NPY_BOOL, npy_bool,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_UBYTE, npy_ubyte,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_USHORT, npy_ushort,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_UINT, npy_uint,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_ULONG, npy_ulong,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_ULONGLONG, npy_ulonglong,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_BYTE, npy_byte,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_SHORT, npy_short,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_INT, npy_int,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_LONG, npy_long,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_LONGLONG, npy_longlong,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_FLOAT, npy_float,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_DOUBLE, npy_double,
-                                      coeff, pi, idx);
-                default:
-                    NPY_END_THREADS;
-                    PyErr_SetString(PyExc_RuntimeError,
-                                    "data type not supported");
-                    goto exit;
+                if (is_cval) {
+                    coeff = cval;
+                } else {
+                    if (NPY_UNLIKELY(edge)) {
+                        for(ll = 0; ll < irank; ll++) {
+                            if (edge_offsets[ll])
+                                idx += edge_offsets[ll][ff[ll]];
+                            else
+                                idx += ff[ll] * istrides[ll];
+                        }
+                    } else {
+                        idx = foffsets[hh];
+                    }
+                    idx += offset;
+                    switch (type_num) {
+                        CASE_INTERP_COEFF(NPY_BOOL, npy_bool,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_UBYTE, npy_ubyte,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_USHORT, npy_ushort,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_UINT, npy_uint,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_ULONG, npy_ulong,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_ULONGLONG, npy_ulonglong,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_BYTE, npy_byte,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_SHORT, npy_short,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_INT, npy_int,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_LONG, npy_long,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_LONGLONG, npy_longlong,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_FLOAT, npy_float,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_DOUBLE, npy_double,
+                                          coeff, pi, idx);
+                    default:
+                        NPY_END_THREADS;
+                        PyErr_SetString(PyExc_RuntimeError,
+                                        "data type not supported");
+                        goto exit;
+                    }
                 }
                 /* calculate the interpolated value: */
                 for(ll = 0; ll < irank; ll++)
@@ -544,6 +619,11 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
  exit:
     NPY_END_THREADS;
     free(edge_offsets);
+    if (edge_grid_const) {
+        for(jj = 0; jj < irank; jj++)
+            free(edge_grid_const[jj]);
+        free(edge_grid_const);
+    }
     if (data_offsets) {
         for(jj = 0; jj < irank; jj++)
             free(data_offsets[jj]);
@@ -561,7 +641,7 @@ NI_GeometricTransform(PyArrayObject *input, int (*map)(npy_intp*, double*,
 
 int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                  PyArrayObject* shift_ar, PyArrayObject *output,
-                 int order, int mode, double cval)
+                 int order, int mode, double cval, int nprepad, int grid_mode)
 {
     char *po, *pi;
     npy_intp **zeros = NULL, **offsets = NULL, ***edge_offsets = NULL;
@@ -569,6 +649,7 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     npy_intp jj, hh, kk, filter_size, odimensions[NPY_MAXDIMS];
     npy_intp idimensions[NPY_MAXDIMS], istrides[NPY_MAXDIMS];
     npy_intp size;
+    char ***edge_grid_const = NULL;
     double ***splvals = NULL;
     NI_Iterator io;
     npy_double *zooms = zoom_ar ? (npy_double*)PyArray_DATA(zoom_ar) : NULL;
@@ -603,15 +684,36 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                 goto exit;
             }
         }
+    } else if (mode == NI_EXTEND_GRID_CONSTANT) {
+        // boolean indicating if the current point in the filter footprint is
+        // outside the bounds
+        edge_grid_const = malloc(rank * sizeof(char*));
+        if (NPY_UNLIKELY(!edge_grid_const)) {
+            NPY_END_THREADS;
+            PyErr_NoMemory();
+            goto exit;
+        }
+        for(jj = 0; jj < rank; jj++)
+            edge_grid_const[jj] = NULL;
+        for(jj = 0; jj < rank; jj++) {
+            edge_grid_const[jj] = malloc(odimensions[jj] * sizeof(char*));
+            if (NPY_UNLIKELY(!edge_grid_const[jj])) {
+                NPY_END_THREADS;
+                PyErr_NoMemory();
+                goto exit;
+            }
+            for(hh = 0; hh < odimensions[jj]; hh++) {
+                edge_grid_const[jj][hh] = NULL;
+            }
+        }
     }
-
     /* store offsets, along each axis: */
     offsets = malloc(rank * sizeof(npy_intp*));
     /* store spline coefficients, along each axis: */
     splvals = malloc(rank * sizeof(double**));
     /* store offsets at all edges: */
-    edge_offsets = malloc(rank * sizeof(npy_intp**));
-    if (NPY_UNLIKELY(!offsets || !splvals || !edge_offsets)) {
+
+    if (NPY_UNLIKELY(!offsets || !splvals)) {
         NPY_END_THREADS;
         PyErr_NoMemory();
         goto exit;
@@ -619,24 +721,45 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
     for(jj = 0; jj < rank; jj++) {
         offsets[jj] = NULL;
         splvals[jj] = NULL;
-        edge_offsets[jj] = NULL;
     }
     for(jj = 0; jj < rank; jj++) {
         offsets[jj] = malloc(odimensions[jj] * sizeof(npy_intp));
         splvals[jj] = malloc(odimensions[jj] * sizeof(double*));
-        edge_offsets[jj] = malloc(odimensions[jj] * sizeof(npy_intp*));
-        if (NPY_UNLIKELY(!offsets[jj] || !splvals[jj] || !edge_offsets[jj])) {
+        if (NPY_UNLIKELY(!offsets[jj] || !splvals[jj])) {
             NPY_END_THREADS;
             PyErr_NoMemory();
             goto exit;
         }
         for(hh = 0; hh < odimensions[jj]; hh++) {
             splvals[jj][hh] = NULL;
-            edge_offsets[jj][hh] = NULL;
         }
     }
 
-    /* precalculate offsets, and offsets at the edge: */
+    if (mode != NI_EXTEND_GRID_CONSTANT){
+        edge_offsets = malloc(rank * sizeof(npy_intp**));
+        if (NPY_UNLIKELY(!edge_offsets)) {
+            NPY_END_THREADS;
+            PyErr_NoMemory();
+            goto exit;
+        }
+        for(jj = 0; jj < rank; jj++) {
+            edge_offsets[jj] = NULL;
+        }
+        for(jj = 0; jj < rank; jj++) {
+            edge_offsets[jj] = malloc(odimensions[jj] * sizeof(npy_intp*));
+            if (NPY_UNLIKELY(!edge_offsets[jj])) {
+                NPY_END_THREADS;
+                PyErr_NoMemory();
+                goto exit;
+            }
+            for(hh = 0; hh < odimensions[jj]; hh++) {
+                edge_offsets[jj][hh] = NULL;
+            }
+        }
+    }
+
+    int spline_mode = _get_spline_boundary_mode(mode);
+
     for(jj = 0; jj < rank; jj++) {
         double shift = 0.0, zoom = 0.0;
         if (shifts)
@@ -648,9 +771,22 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
             if (shifts)
                 cc += shift;
             if (zooms)
-                cc *= zoom;
-            cc = map_coordinate(cc, idimensions[jj], mode);
-            if (cc > -1.0) {
+            {
+                if (grid_mode)
+                {
+                    cc += 0.5;
+                    cc *= zoom;
+                    cc -= 0.5;
+                } else {
+                    cc *= zoom;
+                }
+            }
+            cc += (double)nprepad;
+            if ((mode != NI_EXTEND_GRID_CONSTANT) && (mode != NI_EXTEND_NEAREST)) {
+                /* if the input coordinate is outside the borders, map it: */
+                cc = map_coordinate(cc, idimensions[jj], mode);
+            }
+            if (cc > -1.0 || mode == NI_EXTEND_GRID_CONSTANT || mode == NI_EXTEND_NEAREST) {
                 npy_intp start;
                 if (zeros && zeros[jj])
                     zeros[jj][kk] = 0;
@@ -661,30 +797,33 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                 }
                 offsets[jj][kk] = istrides[jj] * start;
                 if (start < 0 || start + order >= idimensions[jj]) {
-                    edge_offsets[jj][kk] = malloc((order + 1) * sizeof(npy_intp));
-                    if (NPY_UNLIKELY(!edge_offsets[jj][kk])) {
-                        NPY_END_THREADS;
-                        PyErr_NoMemory();
-                        goto exit;
-                    }
-                    for(hh = 0; hh <= order; hh++) {
-                        npy_intp idx = start + hh;
-                        npy_intp len = idimensions[jj];
-                        if (len <= 1) {
-                            idx = 0;
-                        } else {
-                            npy_intp s2 = 2 * len - 2;
-                            if (idx < 0) {
-                                idx = s2 * (npy_intp)(-idx / s2) + idx;
-                                idx = idx <= 1 - len ? idx + s2 : -idx;
-                            } else if (idx >= len) {
-                                idx -= s2 * (npy_intp)(idx / s2);
-                                if (idx >= len)
-                                    idx = s2 - idx;
-                            }
+                    npy_intp idx = 0;
+
+                    if (mode == NI_EXTEND_GRID_CONSTANT) {
+                        edge_grid_const[jj][kk] = malloc((order + 1) * sizeof(char));
+                        if (NPY_UNLIKELY(!edge_grid_const[jj][kk])) {
+                            NPY_END_THREADS;
+                            PyErr_NoMemory();
+                            goto exit;
+                        }
+                        for(hh = 0; hh <= order; hh++) {
+                            idx = start + hh;
+                            edge_grid_const[jj][kk][hh] = (idx < 0 || idx >= idimensions[jj]);
+                        }
+                    } else {
+                        edge_offsets[jj][kk] = malloc((order + 1) * sizeof(npy_intp));
+                        if (NPY_UNLIKELY(!edge_offsets[jj][kk])) {
+                            NPY_END_THREADS;
+                            PyErr_NoMemory();
+                            goto exit;
+                        }
+                        for(hh = 0; hh <= order; hh++) {
+                            idx = start + hh;
+                            idx = (npy_intp)map_coordinate(idx, idimensions[jj], spline_mode);
+                            edge_offsets[jj][kk][hh] = istrides[jj] * (idx - start);
                         }
-                        edge_offsets[jj][kk][hh] = istrides[jj] * (idx - start);
                     }
+
                 }
                 if (order > 0) {
                     splvals[jj][kk] = malloc((order + 1) * sizeof(double));
@@ -750,9 +889,10 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
                 break;
             }
             oo += offsets[hh][io.coordinates[hh]];
-            if (edge_offsets[hh][io.coordinates[hh]])
-                edge = 1;
-        }
+            if (mode != NI_EXTEND_GRID_CONSTANT) {
+                if (edge_offsets[hh][io.coordinates[hh]])
+                    edge = 1;
+            }         }
 
         if (!zero) {
             npy_intp *ff = fcoordinates;
@@ -761,52 +901,66 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
             for(hh = 0; hh < filter_size; hh++) {
                 npy_intp idx = 0;
                 double coeff = 0.0;
-
-                if (NPY_UNLIKELY(edge)) {
-                    /* use precalculated edge offsets: */
+                int is_cval = 0;
+                if (mode == NI_EXTEND_GRID_CONSTANT)
+                {
                     for(jj = 0; jj < rank; jj++) {
-                        if (edge_offsets[jj][io.coordinates[jj]])
-                            idx += edge_offsets[jj][io.coordinates[jj]][ff[jj]];
-                        else
-                            idx += ff[jj] * istrides[jj];
+                        if (edge_grid_const[jj][io.coordinates[jj]])
+                        {
+                            if (edge_grid_const[jj][io.coordinates[jj]][ff[jj]])
+                                is_cval = 1;
+                        }
                     }
-                    idx += oo;
-                } else {
-                    /* use normal offsets: */
-                    idx += oo + foffsets[hh];
                 }
-                switch (type_num) {
-                    CASE_INTERP_COEFF(NPY_BOOL, npy_bool,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_UBYTE, npy_ubyte,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_USHORT, npy_ushort,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_UINT, npy_uint,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_ULONG, npy_ulong,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_ULONGLONG, npy_ulonglong,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_BYTE, npy_byte,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_SHORT, npy_short,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_INT, npy_int,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_LONG, npy_long,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_LONGLONG, npy_longlong,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_FLOAT, npy_float,
-                                      coeff, pi, idx);
-                    CASE_INTERP_COEFF(NPY_DOUBLE, npy_double,
-                                      coeff, pi, idx);
-                default:
-                    NPY_END_THREADS;
-                    PyErr_SetString(PyExc_RuntimeError,
-                                    "data type not supported");
-                    goto exit;
+                if (is_cval) {
+                    coeff = cval;
+                } else {
+                    if (NPY_UNLIKELY(edge)) {
+                        /* use precalculated edge offsets: */
+                        for(jj = 0; jj < rank; jj++) {
+                            if (edge_offsets[jj][io.coordinates[jj]])
+                                idx += edge_offsets[jj][io.coordinates[jj]][ff[jj]];
+                            else
+                                idx += ff[jj] * istrides[jj];
+                        }
+                        idx += oo;
+                    } else {
+                        /* use normal offsets: */
+                        idx += oo + foffsets[hh];
+                    }
+                    switch (type_num) {
+                        CASE_INTERP_COEFF(NPY_BOOL, npy_bool,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_UBYTE, npy_ubyte,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_USHORT, npy_ushort,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_UINT, npy_uint,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_ULONG, npy_ulong,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_ULONGLONG, npy_ulonglong,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_BYTE, npy_byte,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_SHORT, npy_short,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_INT, npy_int,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_LONG, npy_long,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_LONGLONG, npy_longlong,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_FLOAT, npy_float,
+                                          coeff, pi, idx);
+                        CASE_INTERP_COEFF(NPY_DOUBLE, npy_double,
+                                          coeff, pi, idx);
+                    default:
+                        NPY_END_THREADS;
+                        PyErr_SetString(PyExc_RuntimeError,
+                                        "data type not supported");
+                        goto exit;
+                    }
                 }
                 /* calculate interpolated value: */
                 for(jj = 0; jj < rank; jj++)
@@ -873,6 +1027,16 @@ int NI_ZoomShift(PyArrayObject *input, PyArrayObject* zoom_ar,
         }
         free(edge_offsets);
     }
+    if (edge_grid_const) {
+        for(jj = 0; jj < rank; jj++) {
+            if (edge_grid_const[jj]) {
+                for(hh = 0; hh < odimensions[jj]; hh++)
+                    free(edge_grid_const[jj][hh]);
+                free(edge_grid_const[jj]);
+            }
+        }
+        free(edge_grid_const);
+    }
     free(foffsets);
     free(fcoordinates);
     return PyErr_Occurred() ? 0 : 1;
